﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Tree
{
    public partial class Form1 : Form
    {
        private TreeNode root;
        private TreeNode now;

        public Form1()
        {
            InitializeComponent();

            // 先建立一個 root, 更新資料。
            now = root = new TreeNode('R', null);
            UpdateButtomState();
            Invalidate();
        }

        private void DrawTree(TreeNode node, Graphics g, PointF p, double length, double angle)
        {
            // 算出下一層的角度與長度
            double nextAngle = angle * 0.6 > Math.PI / 12 ? angle * 0.6 : Math.PI / 12;
            double nextLength = length * 0.6 > 30 ? length * 0.6 : 30;

            // 如果 left 存在，畫線並且遞迴呼叫，處理下一層
            if (node.left != null)
            {
                PointF l = new PointF((float)(p.X - Math.Abs(length * Math.Sin(angle))), (float)(p.Y + Math.Abs(length * Math.Cos(angle))));
                g.DrawLine(Pens.Black, new PointF(p.X + 7, p.Y), l);
                DrawTree(node.left, g, l, nextLength, nextAngle);
            }

            // 如果 right 存在，畫線並且遞迴呼叫，處理下一層
            if (node.right != null)
            {
                PointF r = new PointF((float)(p.X + Math.Abs(length * Math.Sin(angle))), (float)(p.Y + Math.Abs(length * Math.Cos(angle))));
                g.DrawLine(Pens.Black, new PointF(p.X + 7, p.Y), r);
                DrawTree(node.right, g, r, nextLength, nextLength);
            }

            // 如果 node 跟 now 是同一個點，畫紅色圓
            if (node == now)
                DrawCircleByCenterRadius(g, p, 12, Color.Red);

            // 否則畫黑色圓
            else
                DrawCircleByCenterRadius(g, p, 12, Color.Black);

            // 把那個英文字畫上去
            g.DrawString(node.data.ToString(), new Font("新細明體", 10),
                Brushes.Black, p);
        }

        // 藉由傳入的點、半徑與顏色畫圓
        private void DrawCircleByCenterRadius(Graphics g, PointF p, int r, Color c)
        {
            // 先畫一個填滿白色的實心圓，可以把多餘的線條蓋掉
            g.FillEllipse(Brushes.White, p.X - r / 2, p.Y - r / 2, 2 * r, 2 * r);

            // 再畫上空心圓，顏色用在這裡
            g.DrawEllipse(new Pen(c), p.X - r / 2, p.Y - r / 2, 2 * r, 2 * r);
        }

        // 控制各個按鈕的 Enable 或 Disable
        private void UpdateButtomState()
        {
            textBox1.Text = now.data.ToString();

            LeftGo.Enabled  = now.left != null;
            LeftNew.Enabled = now.left == null;
            LeftDel.Enabled = now.left != null && now.left.left == null && now.left.right == null;

            RightGo.Enabled  = now.right != null;
            RightNew.Enabled = now.right == null;
            RightDel.Enabled = now.right != null && now.right.left == null && now.right.right == null;

            ParentGo.Enabled = now.parent != null;
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            if (root != null)
                DrawTree(root, e.Graphics, new PointF(pictureBox1.Size.Width / 2, 25), 150, Math.PI / 2.3);
        }

        // 可以藉由 TextBox 去修改 node 的內容
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            now.data = textBox1.Text[0];
            UpdateButtomState();
            pictureBox1.Invalidate();
        }

        #region 七個按鈕的處理函式
        private void LeftGo_Click(object sender, EventArgs e)
        {
            // 移到左節點
            now = now.left;
            UpdateButtomState();
            pictureBox1.Invalidate();
        }

        private void LeftNew_Click(object sender, EventArgs e)
        {
            // 新增左節點
            Form2 inputDialog = new Form2();
            inputDialog.ShowDialog();
            now.left = new TreeNode(inputDialog.c, now);
            UpdateButtomState();
            pictureBox1.Invalidate();
        }

        private void LeftDel_Click(object sender, EventArgs e)
        {
            // 刪除左節點
            now.left = null;
            UpdateButtomState();
            pictureBox1.Invalidate();
        }

        private void RightGo_Click(object sender, EventArgs e)
        {
            // 移到右節點
            now = now.right;
            UpdateButtomState();
        }

        private void RightNew_Click(object sender, EventArgs e)
        {
            // 新增右節點
            Form2 inputDialog = new Form2();
            inputDialog.ShowDialog();
            now.right = new TreeNode(inputDialog.c, now);
            UpdateButtomState();
            pictureBox1.Invalidate();
        }

        private void RightDel_Click(object sender, EventArgs e)
        {
            // 刪除右節點
            now.right = null;
            UpdateButtomState();
            pictureBox1.Invalidate();
        }

        private void ParentGo_Click(object sender, EventArgs e)
        {
            // 移到父節點
            now = now.parent;
            UpdateButtomState();
            pictureBox1.Invalidate();
        }
        #endregion
    }

    public class TreeNode
    {
        public TreeNode(char i, TreeNode parent)
        {
            this.data = i;
            this.parent = parent;
        }

        public char data;
        public TreeNode parent;
        public TreeNode right;
        public TreeNode left;
    }
}